#include <string.h>
#include <gtk/gtk.h>
#include "guiutils.h"
#include "cdialog.h"
#include "win.h"
#include "windnd.h"
#include "core.h"
#include "config.h"


static void WinDNDDragDataReceivedNexus(
	win_struct *win, GdkDragContext *dc, guint info,
	const guint8 *data, gint data_length
);

void WinDNDDragDataReceivedCB(
	GtkWidget *widget, GdkDragContext *dc,
	gint x, gint y,                       
	GtkSelectionData *selection_data, guint info, guint t,
	gpointer data                                         
);


#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)


/*
 *	Win "drag_data_received" signal nexus.
 */
static void WinDNDDragDataReceivedNexus(
	win_struct *win, GdkDragContext *dc, guint info,
	const guint8 *data, gint data_length
)
{
	if((data == NULL) || (data_length <= 0))
	    return;

	WinSetBusy(win, TRUE);


	/* Handle by data type */
	if((info == DND_INFO_TEXT_PLAIN) ||
	   (info == DND_INFO_TEXT_URI_LIST) ||
	   (info == DND_INFO_STRING)
	)
	{
	    const gchar	*s, *url = (const gchar *)data;
	    gchar *path;

	    s = strstr(url, "://");
	    if(s != NULL)
		s += STRLEN("://");
	    else
		s = url;

	    path = STRDUP(strchr(s, '/'));
	    if(!STRISEMPTY(path))
	    {
		WinOpen(win, path);
		if(win->play_on_open)
		    WinPlay(win);
	    }
	    g_free(path);
	}

	WinSetBusy(win, FALSE);
}


/*
 *	Win "drag_data_received" signal callback.
 */
void WinDNDDragDataReceivedCB(
	GtkWidget *widget, GdkDragContext *dc,
	gint x, gint y,
	GtkSelectionData *selection_data, guint info, guint t,
	gpointer data
)
{
	win_struct *win = WIN(data);
	if((widget == NULL) || (dc == NULL) || (win == NULL))
	    return;

	WinDNDDragDataReceivedNexus(
	    win, dc, info, selection_data->data, selection_data->length
	);
}
